/**
 * CFormItem
 * @author Tevin
 * @tutorial rules see https://github.com/yiminghe/async-validator#type
 */

<template>
    <view class="c-form-item">
        <view class="c-form-item-label">
            <text
                class="required"
                v-if="required"
            >*</text>
            <text>{{label || name}}</text>
        </view>
        <slot
            :value="formData[name]"
            :onChange="evt=>onChange(evt)"
        />
    </view>
</template>

<script>
import Schema from 'async-validator';
import { validateMsgs } from './validateMsgs.js';

export default {
    name: 'CFormItem',
    props: {
        name: String,
        label: String,
        required: Boolean,
        rules: Array,
        formData: Object,
    },
    data() {
        return {};
    },
    methods: {
        onChange(evt) {
            // 当类型为 object 时，必须为简单 object
            if (Object.prototype.toString.call(evt) === '[object Object]') {
                const hasOwn = Object.prototype.hasOwnProperty;
                if (
                    evt.constructor &&
                    !hasOwn.call(evt, 'constructor') &&
                    !hasOwn.call(evt.constructor.prototype, 'isPrototypeOf')
                ) {
                    throw new Error('错误的表单项 onChange 参数类型！（At: ' + this.name + '）');
                    return;
                }
            }
            // 未改变值不触发
            if (this.formData[this.name] === evt) {
                return;
            }
            this.formData.$handleChange({
                [this.name]: evt,
            });
        },
    },
    mounted() {
        this.$nextTick(() => {
            // 未设置验证
            if (!this.required && !this.rules) {
                this.formData.$regItemValidator(this.name, () => {
                    return Promise.resolve({
                        name: this.name,
                        passed: true,
                    });
                });
            } else {
                const descriptor = this.rules || [];
                if (this.required) {
                    descriptor.unshift({
                        required: true,
                    });
                }
                const validator = new Schema({
                    [this.name]: descriptor,
                });
                validator.messages(validateMsgs); // 自定义验证消息
                this.formData.$regItemValidator(this.name, () => {
                    return validator
                        .validate({
                            [this.name]: this.formData[this.name],
                        })
                        .then(
                            (res) => {
                                return {
                                    name: this.name,
                                    passed: true,
                                };
                            },
                            ({ errors, fields }) => {
                                return {
                                    name: this.name,
                                    passed: false,
                                    msg: errors[0].message.replace(
                                        this.name,
                                        this.label || this.name
                                    ),
                                };
                            }
                        );
                });
            }
        });
    },
    beforeDestroy() {
        this.formData.$regItemValidator(this.name, null);
    },
};
</script>